@@ -3,7 +3,7 @@ This is the main module of the package.
 It handles the local enrichment files, as well as the download of enrichment data from remote servers.
 """
 
-import os
+from pathlib import Path
 import shutil
 
 import json
@@ -66,12 +66,7 @@ def enrich(dataset_ref, var_id, geo_buff = None, time_buff = None, depth_request
         None
     """
 
-    try:
-        f = open(biodiv_path + dataset_ref + '-config.json')
-        enrichment_metadata = json.load(f)
-        f.close()
-    except FileNotFoundError:
-        print('Please create an enrichment file first (use geoenrich.enrichmet.create_enrichment_file)')
+    original, enrichment_metadata = load_enrichment_file(dataset_ref)
 
     input_type = enrichment_metadata['input_type']
     enrichments = enrichment_metadata['enrichments']
@@ -82,8 +77,6 @@ def enrich(dataset_ref, var_id, geo_buff = None, time_buff = None, depth_request
         new_enrichment = True
         enrichment_id = len(enrichments)
 
-    original = load_enrichment_file(dataset_ref)
-
     if slice is None:
         to_enrich = original
     else:
@@ -118,7 +111,7 @@ def enrich(dataset_ref, var_id, geo_buff = None, time_buff = None, depth_request
     # Save file
     if new_enrichment and len(indices):
         save_enrichment_config(dataset_ref, enrichment_id, var_id, geo_buff, time_buff, depth_request, downsample)
-    updated.to_csv(biodiv_path + dataset_ref + '.csv')
+    updated.to_csv(str(Path(biodiv_path, dataset_ref + '.csv')))
 
 
 
@@ -142,24 +135,37 @@ def enrich_compute(geodf, var_id, geo_buff, time_buff, downsample):
 
     # Check if local netcdf files already exist
 
-    if  not(os.path.exists(sat_path + var_id + '.nc')) or \
-        not(os.path.exists(sat_path + var_id + '_downloaded.nc')):
+    if  not(Path(sat_path, var_id + '.nc').exists()) or \
+        not(Path(sat_path, var_id + '_downloaded.nc').exists()):
 
         create_nc_calculated(var_id)
 
     # Backup local netCDF files
 
     timestamp = datetime.now().strftime('%d-%H-%M')
-    shutil.copy2(sat_path + var_id + '.nc', sat_path + var_id + '.nc.' + timestamp)
-    shutil.copy2(sat_path + var_id + '_downloaded.nc', sat_path + var_id + '_downloaded.nc.' + timestamp)
+    shutil.copy2(   str(Path(sat_path, var_id + '.nc')),
+                    str(Path(sat_path, var_id + '.nc.' + timestamp)))
+    shutil.copy2(   str(Path(sat_path, var_id + '_downloaded.nc')),
+                    str(Path(sat_path, var_id + '_downloaded.nc.' + timestamp)))
 
     # Load files
 
-    local_ds = nc.Dataset(sat_path + var_id + '.nc.' + timestamp, mode ='r+')
-    bool_ds = nc.Dataset(sat_path + var_id + '_downloaded.nc.' + timestamp, mode ='r+')
+    local_ds = nc.Dataset(str(Path(sat_path, var_id + '.nc.' + timestamp)), mode ='r+')
+    bool_ds = nc.Dataset(str(Path(sat_path, var_id + '_downloaded.nc.' + timestamp)), mode ='r+')
 
     dimdict, var = get_metadata(local_ds, var_id)
 
+    # Open needed datasets (read-only)
+
+    base_datasets = {}
+    cat = get_var_catalog()
+
+    for sec_var_id in var['derived_from']:
+        base_datasets[sec_var_id] = {}
+        base_datasets[sec_var_id]['ds'] = nc.Dataset(str(Path(sat_path, sec_var_id + '.nc')))
+        base_datasets[sec_var_id]['bool_ds'] = nc.Dataset(str(Path(sat_path, sec_var_id + '_downloaded.nc')))
+        base_datasets[sec_var_id]['varname'] = cat[sec_var_id]['varname']
+
     # Add bounds if occurrences
 
     if 'minx' not in geodf.columns:
@@ -170,34 +176,29 @@ def enrich_compute(geodf, var_id, geo_buff, time_buff, downsample):
     # Remove out of timeframe datapoints
 
     if 'time' in dimdict:
-        t1, t2 = min(dimdict['time']['vals']), max(dimdict['time']['vals'])
+        firstvar = var['derived_from'][0]
+        remote_ds = nc.Dataset(cat[firstvar]['url'])
+        dimdict_2, _ = get_metadata(remote_ds, firstvar)
+        remote_ds.close()
+        t1, t2 = min(dimdict_2['time']['vals']), max(dimdict_2['time']['vals'])
         geodf2 = geodf[(geodf['mint'] >= t1) & (geodf['maxt'] <= t2)]
         print('Ignoring {} rows because data is not available at these dates'.format(len(geodf) - len(geodf2)))
     else:
         geodf2 = geodf
 
-
-
-    # Open needed datasets (read-only)
-
-    base_datasets = {}
-    cat = get_var_catalog()
-
-    for sec_var_id in var['derived_from']:
-        base_datasets[sec_var_id] = {}
-        base_datasets[sec_var_id]['ds'] = nc.Dataset(sat_path + sec_var_id + '.nc')
-        base_datasets[sec_var_id]['bool_ds'] = nc.Dataset(sat_path + sec_var_id + '_downloaded.nc')
-        base_datasets[sec_var_id]['varname'] = cat[sec_var_id]['varname']
-
+    if 'time' in dimdict:
+        firstvar = var['derived_from'][0]
+        local_ds.variables[dimdict['time']['name']][:] = base_datasets[firstvar]['ds'].variables[
+                                                             dimdict['time']['name']][:]
+        dimdict, var = get_metadata(local_ds, var_id)
+        
     # Apply query to each row sequentially
 
-
     geodf2['ind'] = geodf2.apply(calculate_indices, axis = 1, args = (dimdict, var, 'surface', downsample))
     res = geodf2.progress_apply(row_compute, axis=1, args = (local_ds, bool_ds, base_datasets,
                                                              dimdict, var, downsample), 
                                 result_type = 'expand')
 
-
     local_ds.close()
     bool_ds.close()
 
@@ -207,11 +208,11 @@ def enrich_compute(geodf, var_id, geo_buff, time_buff, downsample):
 
     # Remove backup
 
-    os.remove(sat_path + var_id + '.nc')
-    os.remove(sat_path + var_id + '_downloaded.nc')
+    Path(sat_path, var_id + '.nc').unlink()
+    Path(sat_path, var_id + '_downloaded.nc').unlink()
 
-    os.rename(sat_path + var_id + '.nc.' + timestamp, sat_path + var_id + '.nc')
-    os.rename(sat_path + var_id + '_downloaded.nc.' + timestamp, sat_path + var_id + '_downloaded.nc')
+    Path(sat_path, var_id + '.nc.' + timestamp).rename(Path(sat_path, var_id + '.nc'))
+    Path(sat_path, var_id + '_downloaded.nc.' + timestamp).rename(Path(sat_path, var_id + '_downloaded.nc'))
 
     return(res)
 
@@ -256,21 +257,23 @@ def enrich_download(geodf, varname, var_id, url, geo_buff, time_buff, depth_requ
 
     # Check if local netcdf files already exist
 
-    if  not(os.path.exists(sat_path + var['var_id'] + '.nc')) or \
-        not(os.path.exists(sat_path + var['var_id'] + '_downloaded.nc')):
+    if  not(Path(sat_path, var_id + '.nc').exists()) or \
+        not(Path(sat_path, var_id + '_downloaded.nc').exists()):
 
         create_nc(get_var_catalog()[var_id])
 
     # Backup local netCDF files
 
     timestamp = datetime.now().strftime('%d-%H-%M')
-    shutil.copy2(sat_path + var['var_id'] + '.nc', sat_path + var['var_id'] + '.nc.' + timestamp)
-    shutil.copy2(sat_path + var['var_id'] + '_downloaded.nc', sat_path + var['var_id'] + '_downloaded.nc.' + timestamp)
+    shutil.copy2(   str(Path(sat_path, var_id + '.nc')),
+                    str(Path(sat_path, var_id + '.nc.' + timestamp)))
+    shutil.copy2(   str(Path(sat_path, var_id + '_downloaded.nc')),
+                    str(Path(sat_path, var_id + '_downloaded.nc.' + timestamp)))
 
     # Load files
 
-    local_ds = nc.Dataset(sat_path + var['var_id'] + '.nc.' + timestamp, mode ='r+')
-    bool_ds = nc.Dataset(sat_path + var['var_id'] + '_downloaded.nc.' + timestamp, mode ='r+')
+    local_ds = nc.Dataset(str(Path(sat_path, var_id + '.nc.' + timestamp)), mode ='r+')
+    bool_ds = nc.Dataset(str(Path(sat_path, var_id + '_downloaded.nc.' + timestamp)), mode ='r+')
 
     # Remove out of timeframe datapoints
 
@@ -294,6 +297,14 @@ def enrich_download(geodf, varname, var_id, url, geo_buff, time_buff, depth_requ
         res = geodf2.progress_apply(row_enrich, axis=1, args = (remote_ds, local_ds, bool_ds, dimdict, var, depth_request, downsample), 
                             result_type = 'expand')
     
+    # Update time variable in local dataset if needed
+    
+    if 'time' in dimdict and local_ds.variables[dimdict['time']['name']][:].mask.any():
+
+        local_ds.variables[dimdict['time']['name']][:] = remote_ds.variables[dimdict['time']['name']][:]
+
+
+    # Close datasets
 
     local_ds.close()
     bool_ds.close()
@@ -302,11 +313,11 @@ def enrich_download(geodf, varname, var_id, url, geo_buff, time_buff, depth_requ
 
     # Remove backup
 
-    os.remove(sat_path + var['var_id'] + '.nc')
-    os.remove(sat_path + var['var_id'] + '_downloaded.nc')
+    Path(sat_path, var_id + '.nc').unlink()
+    Path(sat_path, var_id + '_downloaded.nc').unlink()
 
-    os.rename(sat_path + var['var_id'] + '.nc.' + timestamp, sat_path + var['var_id'] + '.nc')
-    os.rename(sat_path + var['var_id'] + '_downloaded.nc.' + timestamp, sat_path + var['var_id'] + '_downloaded.nc')
+    Path(sat_path, var_id + '.nc.' + timestamp).rename(Path(sat_path, var_id + '.nc'))
+    Path(sat_path, var_id + '_downloaded.nc.' + timestamp).rename(Path(sat_path, var_id + '_downloaded.nc'))
 
     print('Enrichment over')
     return(res)
@@ -616,11 +627,18 @@ def download_data(remote_ds, local_ds, bool_ds, var, dimdict, ind):
         totalsize = totalsize * act_len
         
 
-    if 'time' in ind:
+    if ('time' in ind) and (check.ndim == len(ind)):
 
-        lentime = 1 + (ind['time']['max'] - ind['time']['min']) // ind['time']['step']
-        flatcheck = check.reshape((lentime, -1)).sum(axis = 1)
-        checklist = (flatcheck == (totalsize / lentime))
+        time_pos = var['params'].index(dimdict['time']['name'])
+        expected_lentime = 1 + (ind['time']['max'] - ind['time']['min']) // ind['time']['step']
+        actual_lentime = check.shape[time_pos]
+    
+        if expected_lentime == actual_lentime:
+            #flatcheck = check.reshape((actual_lentime, -1)).sum(axis = 1)
+            flatcheck = np.array([check.take(i,time_pos).sum() for i in range(actual_lentime)])
+            checklist = (flatcheck == (totalsize / actual_lentime))
+        else:
+            checklist = np.array(False)
 
     else:
         checklist = np.array(check.sum() == totalsize)
@@ -639,7 +657,7 @@ def download_data(remote_ds, local_ds, bool_ds, var, dimdict, ind):
         start = 0
         started = False
 
-        for i in range(lentime):
+        for i in range(len(checklist)):
             is_present = checklist[i]
 
             if not(started) and not(is_present):
@@ -656,7 +674,7 @@ def download_data(remote_ds, local_ds, bool_ds, var, dimdict, ind):
         if(started):
             new_ind = deepcopy(ind)
             new_ind['time']['min'] = ind['time']['min'] + start * ind['time']['step']
-            new_ind['time']['max'] = ind['time']['min'] + (lentime - 1) * ind['time']['step']
+            new_ind['time']['max'] = ind['time']['min'] + (len(checklist) - 1) * ind['time']['step']
             download_data(remote_ds, local_ds, bool_ds, var, dimdict, new_ind)
             
 
@@ -711,28 +729,29 @@ def load_enrichment_file(dataset_ref, mute = False):
         mute (bool): Not printing load message if mute is True.
     Returns:
         geopandas.GeoDataFrame or pandas.DataFrame: Data to enrich (including previously added columns).
+        dict: Enrichment metadata
     """
 
 
-    with open(biodiv_path + dataset_ref + '-config.json') as f:
+    with Path(biodiv_path, dataset_ref + '-config.json').open() as f:
         enrichment_metadata = json.load(f)
     input_type = enrichment_metadata['input_type']
 
-    filepath = biodiv_path + dataset_ref + '.csv'
+    filepath = Path(biodiv_path, dataset_ref + '.csv')
 
 
     if input_type == 'occurrence':
-        df = pd.read_csv(filepath, parse_dates = ['eventDate'], infer_datetime_format = True, index_col = 'id')
+        df = pd.read_csv(str(filepath), parse_dates = ['eventDate'], infer_datetime_format = True, index_col = 'id')
         df['geometry'] = df['geometry'].apply(wkt.loads)
         df = gpd.GeoDataFrame(df, crs = 'epsg:4326')
 
     else:
-        df = pd.read_csv(filepath, parse_dates = ['mint', 'maxt'], infer_datetime_format = True, index_col = 'id')
+        df = pd.read_csv(str(filepath), parse_dates = ['mint', 'maxt'], infer_datetime_format = True, index_col = 'id')
 
     if not(mute):
         print('{} {}s were loaded from enrichment file'.format(len(df), input_type))
     
-    return(df)
+    return(df, enrichment_metadata)
 
 
 
@@ -750,15 +769,15 @@ def create_enrichment_file(gdf, dataset_ref):
         None
     """
 
-    filepath = biodiv_path + dataset_ref + '.csv'
-    filepath_json = biodiv_path + dataset_ref + '-config.json'
+    filepath = Path(biodiv_path, dataset_ref + '.csv')
+    filepath_json = Path(biodiv_path, dataset_ref + '-config.json')
 
-    if(os.path.exists(filepath)):
-        print('Abort. File already exists at ' + filepath)
+    if filepath.exists():
+        print('Abort. File already exists at ' + str(filepath))
     else:
 
         # Write input data to csv
-        gdf.to_csv(filepath)
+        gdf.to_csv(str(filepath))
 
         # Create config file
         config = {}
@@ -775,10 +794,10 @@ def create_enrichment_file(gdf, dataset_ref):
 
         # Writing json config file 
 
-        with open(filepath_json, 'x') as f:
+        with filepath_json.open('x') as f:
             json.dump(config, f, ensure_ascii=False, indent=4)
 
-        print('File saved at ' + filepath)
+        print('File saved at ' + str(filepath))
 
 
 
@@ -793,19 +812,13 @@ def reset_enrichment_file(dataset_ref, var_ids_to_remove):
     Returns:
         None
     """
-    try:
-        f = open(biodiv_path + dataset_ref + '-config.json')
-        enrichment_metadata = json.load(f)
-        f.close()
-    except FileNotFoundError:
-        print('No enrichment file was found for this dataset')
+
+    df, enrichment_metadata = load_enrichment_file(dataset_ref)
 
     enrichments = enrichment_metadata['enrichments']
     input_type = enrichment_metadata['input_type']
+
     remaining_enrichments = []
-
-    df = load_enrichment_file(dataset_ref)
-
     to_drop = []
 
     for enrichment in enrichments:
@@ -825,7 +838,7 @@ def reset_enrichment_file(dataset_ref, var_ids_to_remove):
 
     enrichment_metadata['enrichments'] = remaining_enrichments
 
-    with open(biodiv_path + dataset_ref + '-config.json', 'w') as f:
+    with Path(biodiv_path, dataset_ref + '-config.json').open('w') as f:
         json.dump(enrichment_metadata, f, ensure_ascii=False, indent=4)
 
     print('Enrichment file for dataset ' + dataset_ref + ' was reset.')
@@ -844,14 +857,11 @@ def enrichment_status(dataset_ref):
         pandas.DataFrame: A table of variables and statuses of enrichment.
     """
 
-    with open(biodiv_path + dataset_ref + '-config.json') as f:
-        enrichment_metadata = json.load(f)
+    df, enrichment_metadata = load_enrichment_file(dataset_ref)
 
     enrichments = enrichment_metadata['enrichments']
     input_type = enrichment_metadata['input_type']
 
-    df = load_enrichment_file(dataset_ref)
-
     col_indices = parse_columns(df)
 
     params_dict = {en['id']:en['parameters'] for en in enrichments}
@@ -958,7 +968,7 @@ def save_enrichment_config(dataset_ref, enrichment_id, var_id, geo_buff, time_bu
         None
     """
 
-    with open(biodiv_path + dataset_ref + '-config.json') as f:
+    with Path(biodiv_path, dataset_ref + '-config.json').open() as f:
         enrichment_metadata = json.load(f)
 
     new_enrichment = {'id': enrichment_id,
@@ -973,7 +983,7 @@ def save_enrichment_config(dataset_ref, enrichment_id, var_id, geo_buff, time_bu
 
     enrichment_metadata['enrichments'].append(new_enrichment)
 
-    with open(biodiv_path + dataset_ref + '-config.json', 'w') as f:
+    with Path(biodiv_path, dataset_ref + '-config.json').open('w') as f:
         json.dump(enrichment_metadata, f, ensure_ascii=False, indent=4)
 
 
@@ -989,7 +999,7 @@ def read_ids(dataset_ref):
         list: List of all present ids.
     """
 
-    filepath = biodiv_path + dataset_ref + '.csv'
-    df = pd.read_csv(filepath, index_col = 'id')
+    filepath = Path(biodiv_path, dataset_ref + '.csv')
+    df = pd.read_csv(str(filepath), index_col = 'id')
 
     return(list(df.index))
