@@ -6,11 +6,15 @@ After enriching occurrences, you can use the exports module to use the downloade
 - Retrieve the raw data as a numpy array with :func:`geoenrich.exports.retrieve data`.
 
 """
+from pathlib import Path
 import json
 import pandas as pd
 import cv2
 from matplotlib import cm
 
+import rasterio
+from rasterio.transform import from_origin
+
 import geoenrich
 
 try:
@@ -23,8 +27,8 @@ from geoenrich.satellite import *
 
 
 
-def retrieve_data(dataset_ref, occ_id, var_id, geo_buff = None, time_buff = None, depth_request = 'surface',
-                    downsample = {}, shape = 'rectangle', df = None):
+def retrieve_data(dataset_ref, occ_id, var_id, geo_buff = None, time_buff = None, depth_request = None,
+                    downsample = None, shape = 'rectangle', df = None):
 
     """
     Retrieve downloaded data for the given occurrence id and variable.
@@ -45,14 +49,14 @@ def retrieve_data(dataset_ref, occ_id, var_id, geo_buff = None, time_buff = None
     """
 
 
-    with open(biodiv_path + dataset_ref + '-config.json') as f:
+    with Path(biodiv_path, dataset_ref + '-config.json').open() as f:
         enrichment_metadata = json.load(f)
 
     enrichments = enrichment_metadata['enrichments']
     input_type = enrichment_metadata['input_type']
 
     if df is None:
-        df = load_enrichment_file(dataset_ref, mute = True)
+        df, _ = load_enrichment_file(dataset_ref, mute = True)
     row = df.loc[occ_id]
 
     # Identify relevant enrichment ids
@@ -86,7 +90,7 @@ def retrieve_data(dataset_ref, occ_id, var_id, geo_buff = None, time_buff = None
             results = {'coords': None, 'values': None}
 
         else:
-            ds = nc.Dataset(sat_path + var_id + '.nc')
+            ds = nc.Dataset(str(Path(sat_path, var_id + '.nc')))
             unit = getattr(ds.variables[var_source['varname']], 'units', 'Unspecified')
 
             dimdict, var = get_metadata(ds, var_source['varname'])
@@ -133,6 +137,9 @@ def fetch_data(row, var_id, var_indices, ds, dimdict, var, downsample, indices =
     else:
         ordered_indices = [indices[p] for p in params]
 
+    if downsample is None:
+        downsample = {}
+
     for i in range(len(params)):
         p = params[i]
         if p in downsample:
@@ -158,7 +165,10 @@ def fetch_data(row, var_id, var_indices, ds, dimdict, var, downsample, indices =
             if 'months since' in time_var.__dict__['units']:
                 times = num2date(time_var[i1:i2+1:step], time_var.__dict__['units'], '360_day')
             else:
-                times = num2pydate(time_var[i1:i2+1:step], time_var.__dict__['units'])
+                if var['name'] in ['uwnd', 'vwnd']:
+                    times = num2pydate(time_var[i1:i2+1:step] - 725563, 'days since 1987-01-01 00:00:00')
+                else:
+                    times = num2pydate(time_var[i1:i2+1:step], time_var.__dict__['units'])
             coordinates.append([p, times])
         else:
             coordinates.append([p, ds.variables[dimdict[p]['name']][i1:i2+1:step]])
@@ -168,8 +178,8 @@ def fetch_data(row, var_id, var_indices, ds, dimdict, var, downsample, indices =
 
 
 
-def produce_stats(dataset_ref, var_id, geo_buff = None, time_buff = None, depth_request = 'surface',
-                    downsample = {}, out_path = biodiv_path):
+def produce_stats(dataset_ref, var_id, geo_buff = None, time_buff = None, depth_request = None,
+                    downsample = None, out_path = Path('./')):
 
     """
     Produce a document named *dataset\_ref*\_stats.csv with summary stats of all enriched data.
@@ -182,19 +192,16 @@ def produce_stats(dataset_ref, var_id, geo_buff = None, time_buff = None, depth_
         time_buff (float list): (Optional) Time_buff that was used for enrichment.
         depth_request (str): (Optional) Depth request that was used for enrichment.
         downsample (dict): (Optional) Downsample that was used for enrichment.
-        out_path (str): Path where you want to save the output stats file.
+        out_path (str or pathlib.Path): Path where you want to save the output stats file.
 
     Returns:
         None
     """
 
-    with open(biodiv_path + dataset_ref + '-config.json') as f:
-        enrichment_metadata = json.load(f)
-
+    df, enrichment_metadata = load_enrichment_file(dataset_ref)
     enrichments = enrichment_metadata['enrichments']
     input_type = enrichment_metadata['input_type']
 
-    df = load_enrichment_file(dataset_ref)
 
     # Identify relevant enrichment ids
 
@@ -203,8 +210,8 @@ def produce_stats(dataset_ref, var_id, geo_buff = None, time_buff = None, depth_
         if en['parameters']['var_id'] == var_id:
             if  (geo_buff is None or en['parameters']['geo_buff'] == geo_buff)                  and \
                 (time_buff is None or en['parameters']['time_buff'] == time_buff)               and \
-                (depth_request == 'surface' or en['parameters']['depth_request'] == depth_request)   and \
-                (en['parameters']['downsample'] == downsample):
+                (depth_request is None or en['parameters']['depth_request'] == depth_request)   and \
+                (downsample is None or en['parameters']['downsample'] == downsample):
 
                 relevant.append(en)
 
@@ -224,7 +231,7 @@ def produce_stats(dataset_ref, var_id, geo_buff = None, time_buff = None, depth_
         results = {}
 
         var_source = get_var_catalog()[var_id]
-        ds = nc.Dataset(sat_path + var_id + '.nc')
+        ds = nc.Dataset(str(Path(sat_path, var_id + '.nc')))
         dimdict, var = get_metadata(ds, var_source['varname'])
 
         print('Computing stats for ' + var_id + '...')
@@ -233,9 +240,9 @@ def produce_stats(dataset_ref, var_id, geo_buff = None, time_buff = None, depth_
         ds.close()
 
 
-        filepath = out_path + dataset_ref + '_' + str(en['id']) + '_stats.csv'
-        res.to_csv(filepath)
-        print('File saved at ' + filepath)
+        filepath = Path(out_path, dataset_ref + '_' + str(en['id']) + '_stats.csv')
+        res.to_csv(str(filepath))
+        print(f'File saved at {filepath}')
 
 
 
@@ -314,11 +321,11 @@ def get_derivative(dataset_ref, occ_id, var_id, days = (0,0), geo_buff = None, d
         dict: A dictionary of all available variables with corresponding data (numpy.ma.MaskedArray), unit (str), and coordinates (ordered list of dimension names and values).
     """
 
-    with open(biodiv_path + dataset_ref + '-config.json') as f:
-        enrichment_metadata = json.load(f)
+    df, enrichment_metadata = load_enrichment_file(dataset_ref)
+
     input_type = enrichment_metadata['input_type']
 
-    row = load_enrichment_file(dataset_ref).loc[[occ_id]]
+    row = df.loc[[occ_id]]
     row1, row2 = deepcopy(row), deepcopy(row)
     
     
@@ -338,7 +345,7 @@ def get_derivative(dataset_ref, occ_id, var_id, days = (0,0), geo_buff = None, d
     var_source = get_var_catalog()[var_id]
 
 
-    ds = nc.Dataset(sat_path + var_id + '.nc')
+    ds = nc.Dataset(str(Path(sat_path, var_id + '.nc')))
     unit = getattr(ds.variables[var_source['varname']], 'units', 'Unspecified')
     dimdict, var = get_metadata(ds, var_source['varname'])
 
@@ -363,8 +370,8 @@ def get_derivative(dataset_ref, occ_id, var_id, days = (0,0), geo_buff = None, d
         return({'coords': coords, 'values': data, 'unit': unit + ' per day'})
 
 
-def export_png(dataset_ref, occ_id, var_id, target_size = None, value_range=None, path = biodiv_path,
-               geo_buff = None, time_buff = None, depth_request = 'surface', downsample = {},
+def export_png(dataset_ref, occ_id, var_id, target_size = None, value_range=None, path = Path('./'),
+               geo_buff = None, time_buff = None, depth_request = None, downsample = None,
                cmap = 'coolwarm', shape = 'rectangle'):
 
     """
@@ -378,7 +385,7 @@ def export_png(dataset_ref, occ_id, var_id, target_size = None, value_range=None
         var_id (str): ID of the variable to retrieve.
         target_size (int tuple): Size of the target picture (width, height). If None, using the native data resolution.
         value_range (float list): Range of the variable. Necessary for consistency between all images.
-        path (str): Path where image files will be saved.
+        path (str or pathlib.Path): Path where image files will be saved.
         geo_buff (int): (Optional) Geo_buff that was used for enrichment.
         time_buff (float list): (Optional) Time_buff that was used for enrichment.
         depth_request (str): (Optional) Depth request that was used for enrichment.
@@ -390,12 +397,13 @@ def export_png(dataset_ref, occ_id, var_id, target_size = None, value_range=None
         None
     """
 
-    folderpath = path + dataset_ref + '/'
-    if not(os.path.exists(folderpath)):
-        os.mkdir(folderpath)
+    folderpath = Path(path) / dataset_ref
+    if not folderpath.exists():
+        folderpath.mkdir()
 
     # Retrieve data
-    res = retrieve_data(dataset_ref, occ_id, var_id, geo_buff, time_buff, downsample = downsample, shape=shape)
+    res = retrieve_data(dataset_ref, occ_id, var_id, geo_buff, time_buff, depth_request=depth_request,
+                        downsample = downsample, shape=shape)
 
     if res is not None:
 
@@ -415,25 +423,26 @@ def export_png(dataset_ref, occ_id, var_id, target_size = None, value_range=None
         im3 = cv2.cvtColor(np.float32(im2), cv2.COLOR_BGR2RGBA)
         im3[:,:,3] =  1 - np.isnan(im)
 
-        im_path = folderpath + str(occ_id) + '_' + var_id + '.png'
-        cv2.imwrite(im_path, 255*im3)
-        print('Image saved at ' + im_path)
+        im_path = Path(folderpath, str(occ_id) + '_' + var_id + '.png')
+        cv2.imwrite(str(im_path), 255*im3)
+
+        print(f'Image saved at {im_path}')
 
 
 
-def export_to_array(res, target_size=None, value_range=None, stack=False, squeeze=True, target_len=None):
+def export_to_array(res, target_size=None, value_range=None, stack=None, squeeze=True, target_len=None):
 
     """
     Export data as a 3D numpy array where the first 2 dimensions represent geographical coordinates.
     Option to standardize data by specifiying target size and target value range.
-    The third dimensions stores multiples bands if stack is set to *True* and multiple depth or time values are present.
+    The third dimensions stores multiples bands if stack is set to *depth*, *time* or *all*.
 
 
     Args:
         res (dict): output of :func:`geoenrich.exports.retrieve_data`.
         target_size (int tuple): Size of the target array (width, height). If None, using the native data resolution.
         value_range (float list): Range of the variable. Necessary for consistency between all images.
-        stack (bool): If True, keep values for all depths and times (returns 3D array).
+        stack (str): If True, keep values for all depths or all times (returns 3D array).
         squeeze (bool): If true, remove unused dimensions in the output.
         target_len (int): Length of the third dimension if data is None (to return uniform results).
     Returns:
@@ -445,20 +454,30 @@ def export_to_array(res, target_size=None, value_range=None, stack=False, squeez
 
         im = res['values']
         params = [c[0] for c in res['coords']]
-
-        # Transform to 2D data by removing additional dimensions.
         lat_ax = params.index('latitude')
         lon_ax = params.index('longitude')
 
-        # Transpose if needed
         im1 = deepcopy(im)
 
-        if lat_ax > lon_ax:
-            im1 = np.transpose(im1)
-            lat_ax = params.index('longitude')
-            lon_ax = params.index('latitude')
 
-        if not (stack):
+        # Reorder axes
+
+        if lat_ax != 0:
+            im1 = np.swapaxes(im1, lat_ax, 0)
+            params[0], params[lat_ax] = params[lat_ax], params[0]
+            lat_ax = 0
+
+        if lon_ax != 1:
+            im1 = np.swapaxes(im1, lon_ax, 1)
+            params[1], params[lon_ax] = params[lon_ax], params[1]
+            lon_ax = 1
+
+        # Remove unwanted dimensions
+
+        if stack is None:
+
+            # Keep only shallowest and most recent
+
             if 'time' in params:
                 time_ax = params.index('time')
                 im1 = im1.take(-1, axis=time_ax)
@@ -470,9 +489,36 @@ def export_to_array(res, target_size=None, value_range=None, stack=False, squeez
                 
                 im1 = im1.take(np.argmin(res['coords'][depth_ax][1]), axis=depth_ax)
 
-        im1 = im1.reshape([im.shape[lat_ax], im.shape[lon_ax], -1])
+        elif stack == 'time':
+
+            # Keep all times but only most shallow
+
+            if 'depth' in params:
+                depth_ax = params.index('depth')                
+                im1 = im1.take(np.argmin(res['coords'][depth_ax][1]), axis=depth_ax)
+
+
+        elif stack == 'depth':
+
+            # Keep all depths but only most recent
+
+            if 'time' in params:
+                time_ax = params.index('time')
+                im1 = im1.take(-1, axis=time_ax)
+
+
+        else:
+
+            # Keep everything
+
+            pass
+
+
+        im1 = im1.reshape([im1.shape[0], im1.shape[1], -1])
         mask = im1.mask
 
+        im2 = deepcopy(im1)
+
         # Resize
         if target_size is not None:
 
@@ -508,4 +554,93 @@ def export_to_array(res, target_size=None, value_range=None, stack=False, squeez
     else:
         return (None)
 
-    
\ No newline at end of file
+    
+
+def export_raster(dataset_ref, occ_id, var_id, path = Path('./'), geo_buff = None, time_buff = None,
+                    depth_request = None, downsample = None, shape = 'rectangle', multiband = None):
+
+    """
+    Export a GeoTiff raster of the requested data.
+    Depth or time dimension (not both) can be stored as band (see multiband argument)
+    Otherwise, the shallowest depth and most recent time are selected.
+
+    Args:
+        dataset_ref (str): The enrichment file name (e.g. gbif_taxonKey).
+        occ_id (str): ID of the occurrence to get data for. Can be obtained with :func:`geoenrich.enrichment.read_ids`.
+        var_id (str): ID of the variable to retrieve.
+        path (str or pathlib.Path): Path where image files will be saved.
+        geo_buff (int): (Optional) Geo_buff that was used for enrichment.
+        time_buff (float list): (Optional) Time_buff that was used for enrichment.
+        depth_request (str): (Optional) Depth request that was used for enrichment.
+        downsample (dict): (Optional) Downsample that was used for enrichment.
+        shape (str): If 'rectangle', return data inside the rectangle containing the buffer. If 'buffer', only return data within the buffer distance from the occurrence location.
+        multiband (str): If multiband='depth' or 'time', the corresponding dimension is saved into multiple bands.
+    Returns:
+        None
+    """
+
+    folderpath = Path(path) / (dataset_ref + '_rasters')
+    if not folderpath.exists():
+        folderpath.mkdir()
+
+    # Retrieve data
+    res = retrieve_data(dataset_ref, occ_id, var_id, geo_buff, time_buff, depth_request=depth_request,
+                        downsample = downsample, shape=shape)
+
+    if res is not None:
+
+        assert multiband in [None, 'time', 'depth']
+
+        im = export_to_array(res, target_size=None, value_range=None, stack=multiband)
+
+        # Flip latitude (because image vertical axis is downwards)
+        lat_ax = [c[0] for c in res['coords']].index('latitude')
+        lats = res['coords'][lat_ax][1]
+
+        lon_ax = [c[0] for c in res['coords']].index('longitude')
+        lons = res['coords'][lon_ax][1]
+
+
+        if len(lats)>1 and lats[0] > lats[1]:
+            im = np.flipud(im)
+
+        if im.shape[0] > 1 and im.shape[1] > 1:
+
+            x_pxl = lons[1] - lons[0]
+            y_pxl = lats[0] - lats[1]
+
+            transform = from_origin(lons[0] - .5*x_pxl, lats[0] + .5*y_pxl, x_pxl, y_pxl)
+
+            im_path = Path(folderpath, str(occ_id) + '_' + var_id + '.tiff')
+
+            if len(im.shape) == 2:
+                new_raster = rasterio.open(im_path, 'w', driver='GTiff',
+                                height = im.shape[0], width = im.shape[1],
+                                count=1, dtype=str(im.dtype),
+                                crs='EPSG:4326',
+                                transform=transform)
+
+                new_raster.write(im, 1)
+                new_raster.close()
+            else:
+                band_ax = [c[0] for c in res['coords']].index(multiband)
+                bandnames = res['coords'][band_ax][1]
+
+                new_raster = rasterio.open(im_path, 'w', driver='GTiff',
+                                height = im.shape[0], width = im.shape[1],
+                                count=len(bandnames), dtype=str(im.dtype),
+                                crs='EPSG:4326',
+                                transform=transform)
+
+                for i in range(im.shape[2]):
+                    band = im.take(i, axis=2)
+                    new_raster.write_band(i+1, band)
+                    new_raster.set_band_description(i+1, f'{multiband}={bandnames[i]}')
+
+                new_raster.close()
+
+            print(f'Raster saved at {im_path}')
+
+        else:
+
+            print('Abort. Array is smaller than 2x2 pixels.')
\ No newline at end of file
